// Listing 31.1: Creating a new RSA encryption key pair optional exportability and UI protection

using System;
using System.Security.Cryptography;
using System.Runtime.InteropServices;

public class CreateNonExportableKeyClass 
{
  private static uint CRYPT_NEWKEYSET = 0x00000008;
  
  [DllImport("advapi32.dll",EntryPoint="CryptAcquireContext",CharSet=CharSet.Auto, SetLastError=true)]
  public static extern bool CryptAcquireContext(ref IntPtr hCryptProv, String containerName, 
    String providerName, int providerType, uint providerFlags);

  private static uint CRYPT_EXPORTABLE = 0x00000001;
  private static uint CRYPT_USER_PROTECTED = 0x00000002;
  
  [DllImport("advapi32.dll",EntryPoint="CryptGenKey",CharSet=CharSet.Auto, SetLastError=true)]
  public static extern bool CryptGenKey(IntPtr hProv, uint AlgID, uint dwFlags, ref IntPtr phKey);  
  
  public static void CreateRSAKey(int size, String containerName, bool exportable, bool uiProtected) 
  {
    IntPtr hCryptProv = new IntPtr(0);
    // We assume the key container already exists.  If not, we'll catch the
    // error and try again with CRYPT_NEWKEYSET specified
    uint flags = 0;
    if (exportable) flags |= CRYPT_EXPORTABLE;
    if (uiProtected) flags |= CRYPT_USER_PROTECTED;
    // The desired size of the modulus is contained in the upper 16 bits of 
    // the flags field.  First we check that size is between 384 and 16384 (the
    // size range allowed for Type 1 providers, and then we OR in the bits
    if ((size < 384) || (size > 16384)) 
    {
      throw new ArgumentException("Invalid size parameter.");
    }
    flags |= ((uint) size << 16);
    bool acquireSucceeded = CryptAcquireContext(ref hCryptProv, containerName, null, 1, 0);  
    if (!acquireSucceeded) 
    {
      uint error = (uint) Marshal.GetLastWin32Error();
      // 0x8009000F = NTE_BAD_KEYSET, which means the key container
      // probably doesn't exist, so we'll try to create it
      if (error != 0x80090016) 
      {
        throw new CryptographicException((int) error);  
      }
      acquireSucceeded = CryptAcquireContext(ref hCryptProv, containerName, null, 1, CRYPT_NEWKEYSET);
      if (!acquireSucceeded) 
      {
        throw new CryptographicException(Marshal.GetLastWin32Error());
      }
    }
    IntPtr hCryptKey = new IntPtr(0);
    bool genKeySucceeded = CryptGenKey(hCryptProv, 0x0000A400, flags, ref hCryptKey);
    if (!genKeySucceeded) 
    {
      throw new CryptographicException(Marshal.GetLastWin32Error());
    }
  }


  public static void Main(String[] args) 
  {
    if (args.Length < 1) 
    {
      Console.WriteLine("Usage: createkey [-s size] [-e] [-u] keyContainerName");
      return;
    }
    String keyContainerName = null;
    bool exportable = false;
    bool uiProtected = false;
    int i = 0;
    int size = 1024;
    while (i < args.Length) 
    {
      if (args[i].Equals("-s")) 
      {
        i++;
        size = Int32.Parse(args[i]);
        i++;
        continue;
      }
      if (args[i].Equals("-e")) 
      {
        exportable = true;
        i++;
        continue;
      }
      if (args[i].Equals("-u")) 
      {
        uiProtected = true;
        i++;
        continue;
      }
      keyContainerName = args[i];
      break;
    }
    CreateRSAKey(size, keyContainerName, exportable, uiProtected);
  }
}
